"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _easevalidation = require("easevalidation");

var _oors = require("oors");

var Sentry = _interopRequireWildcard(require("@sentry/node"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

/* eslint-disable class-methods-use-this */
class SentryModule extends _oors.Module {
  constructor(...args) {
    super(...args);
    this.name = 'oors.sentry';
    this.hooks = {
      'oors.express.middlewares': ({
        middlewares
      }) => {
        middlewares.insert(this.getConfig('middlewarePivots').request, {
          id: 'sentryRequestHandler',
          factory: () => Sentry.Handlers.requestHandler()
        });
        middlewares.insert(this.getConfig('middlewarePivots').error, {
          id: 'sentryErrorHandler',
          factory: () => Sentry.Handlers.errorHandler()
        });
      }
    };

    this.logError = error => new Promise(resolve => Sentry.captureException(error, resolve));
  }

  async setup({
    dsn,
    integrations,
    logGqlErrors
  }) {
    Sentry.init({
      dsn,
      integrations
    });

    if (logGqlErrors) {
      this.onModule('oors.graphql', 'error', error => {
        this.logError(error);
      });
    }

    this.exportProperties(['logError']);
  }

}

SentryModule.validateConfig = (0, _easevalidation.validate)(_easevalidation.validators.isSchema({
  dsn: [_easevalidation.validators.isDefault(process.env.SENTRY_DSN), _easevalidation.validators.isRequired(), _easevalidation.validators.isString()],
  integrations: [_easevalidation.validators.isDefault([]), _easevalidation.validators.isArray()],
  middlewarePivots: [_easevalidation.validators.isDefault({}), _easevalidation.validators.isSchema({
    request: [_easevalidation.validators.isRequired(), _easevalidation.validators.isAny(_easevalidation.validators.isString(), _easevalidation.validators.isSchema({
      before: _easevalidation.validators.isAny(_easevalidation.validators.isString(), _easevalidation.validators.isUndefined()),
      after: _easevalidation.validators.isAny(_easevalidation.validators.isString(), _easevalidation.validators.isUndefined())
    }))],
    error: [_easevalidation.validators.isRequired(), _easevalidation.validators.isAny(_easevalidation.validators.isString(), _easevalidation.validators.isSchema({
      before: _easevalidation.validators.isAny(_easevalidation.validators.isString(), _easevalidation.validators.isUndefined()),
      after: _easevalidation.validators.isAny(_easevalidation.validators.isString(), _easevalidation.validators.isUndefined())
    }))]
  })],
  logGqlErrors: [_easevalidation.validators.isDefault(true), _easevalidation.validators.isBoolean()]
}));
var _default = SentryModule;
exports.default = _default;